package com.example.android.iap;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.text.Editable;
import android.text.TextUtils;
import android.view.View;
import android.view.inputmethod.InputMethodManager;
import android.widget.ArrayAdapter;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

import com.example.android.iap.data.PurchaseItem;
import com.example.android.iap.data.QueryItem;
import com.nhnent.mobill.api.exception.InAppExceptionType;
import com.toast.android.iap.InAppPurchase;
import com.toast.android.iap.InAppPurchases;
import com.toast.android.iap.exception.InAppPurchaseException;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.net.HttpURLConnection;
import java.util.ArrayList;

public class MainActivity extends Activity {
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
    }

    /**
     * IAP SDK 를 초기화 합니다.
     * <p>
     * App ID : 'IAP Web Console'에 등록 된 'App ID'. 숫자 7글자로 구성 되어 있습니다.
     * Market ID : 'IAP Web Console'의 'App ID'에 설정 되어 있는 'Store ID'입니다.
     * User ID : 개발사에서 정의한 사용자 식별키이며, 아이템이 지급되는 대상입니다.
     */
    public void clickInitialize(View view) {
        Editable appId = ((EditText) findViewById(R.id.app_id)).getText();
        Editable userId = ((EditText) findViewById(R.id.user_id)).getText();

        if (appId.length() == 0) {
            showMessage(R.string.insert_your_app_id);
        } else if (userId.length() == 0) {
            showMessage(R.string.insert_your_user_id);
        } else {
            String marketId = (String) ((Spinner) findViewById(R.id.market_id)).getSelectedItem();

            //IAP SDK initialize information
            InAppPurchases.InAppPurchase.registerAppId(Long.valueOf(appId.toString()));
            InAppPurchases.InAppPurchase.registerMarketId(marketId);
            InAppPurchases.InAppPurchase.registerUserId(userId.toString());

            //Update status bar
            String initInfo = getString(R.string.initialization_information) +
                    "\nApp ID : " + appId.toString() +
                    ", Store : " + marketId +
                    ", User : " + userId.toString();
            ((TextView) findViewById(R.id.status_bar)).setText(initInfo);

            //Show purchase field
            findViewById(R.id.purchase_layout).setVisibility(View.VISIBLE);

            //Keyboard hide
            InputMethodManager imm = (InputMethodManager) getSystemService(Activity.INPUT_METHOD_SERVICE);
            if (imm != null) {
                imm.hideSoftInputFromWindow(view.getWindowToken(), 0);
            }

            //Guide message
            showMessage(userId.toString() + " " + getString(R.string.initialize_completed));
        }
    }

    /**
     * 구매 가능한 모든 아이템 내역을 조회합니다.
     */
    public void clickQueryItem(View view) {
        showLoadingScreen(true);

        InAppPurchases.InAppPurchase.queryItems(this, new InAppPurchase.ItemListCallback() {
            @Override
            public void onCallback(JSONArray result, InAppPurchaseException exception) {
                showLoadingScreen(false);

                if (exception != null) {
                    handlePurchaseException(exception);
                } else {
                    ArrayList<QueryItem> queryItems = new ArrayList<>();

                    for (int i = 0; i < result.length(); i++) {
                        queryItems.add(new QueryItem(result.optJSONObject(i)));
                    }

                    //TODO : Enter your code
                    showListAlert(R.string.query_items, queryItems);

                    //Guide message
                    showMessage(R.string.press_to_enter_automatically);
                }
            }
        });
    }

    /**
     * 'IAP Web Console'에 등록된 모든 아이템 내역을 조회합니다.
     * 아이템의 상태가 'STOP'인 항목도 모두 표시 됩니다.
     */
    public void clickQueryTcConsoleItem(View view) {
        showLoadingScreen(true);

        InAppPurchases.InAppPurchase.queryTCConsoleItems(this, new InAppPurchase.ItemListCallback() {
            @Override
            public void onCallback(JSONArray result, InAppPurchaseException exception) {
                showLoadingScreen(false);

                if (exception != null) {
                    handlePurchaseException(exception);
                } else {
                    ArrayList<QueryItem> queryItems = new ArrayList<>();

                    for (int i = 0; i < result.length(); i++) {
                        queryItems.add(new QueryItem(result.optJSONObject(i)));
                    }

                    //TODO : Enter your code
                    showListAlert(R.string.query_tc_console_item, queryItems);

                    //Guide message
                    showMessage(R.string.press_to_enter_automatically);
                }
            }
        });
    }

    /**
     * IAP SDK 에서 아이템 구매를 요청합니다.
     * 결제 요청에 대한 응답은 'PurchaseCallback'을 통해 전달 받게 됩니다.
     * 결제가 성공적으로 완료되면 'Payment Consume API' 를 사용하여 결제내역을 소비(Consume) 해야 합니다.
     * 본 예제에서는 'clickConsume' 메소드가 구현되어 있습니다.
     */
    public void clickPurchase(View view) {
        showLoadingScreen(true);

        Editable itemId = ((EditText) findViewById(R.id.item_id)).getText();

        if (itemId.length() == 0) {
            showMessage(R.string.insert_item_id);
        } else if (!itemId.toString().matches("[0-9]+")) {
            showMessage(R.string.enter_only_digits);
        } else {
            InAppPurchases.InAppPurchase.requestPurchase(this, Long.valueOf(itemId.toString()), new InAppPurchase.PurchaseCallback() {

                @Override
                public void onCallback(JSONObject result, InAppPurchaseException exception) {
                    showLoadingScreen(false);

                    if (exception != null) {
                        handlePurchaseException(exception);
                    } else {
                        PurchaseItem purchaseItem = new PurchaseItem(result);

                        //TODO : Enter your code
                        showMessage("PaymentSeq : " + purchaseItem.getPaymentSeq() + "\nItemSeq : " + purchaseItem.getItemSeq() + "\n" + getString(R.string.purchase_completed));
                    }
                }
            });
        }
    }

    /**
     * 유저의 결제가 완료되었으나 소비(consume) 되지 않은 결제내역을 조회합니다.
     * 미소비내역을 조회하여 소비되지 않은 내역들을 소비(consume) 할 수 있습니다
     */
    public void clickQueryPurchases(View view) {
        showLoadingScreen(true);

        InAppPurchases.InAppPurchase.queryPurchases(this, new InAppPurchase.PurchaseListCallback() {

            @Override
            public void onCallback(JSONArray result, InAppPurchaseException exception) {
                showLoadingScreen(false);

                if (exception != null) {
                    handlePurchaseException(exception);
                } else {
                    if (result.length() == 0) {
                        showMessage(R.string.no_query_purchase);
                    } else {
                        ArrayList<PurchaseItem> purchaseItems = new ArrayList<>();
                        for (int i = 0; i < result.length(); i++) {
                            purchaseItems.add(new PurchaseItem(result.optJSONObject(i)));
                        }

                        //TODO : Enter your code
                        showListAlert(R.string.query_purchases, purchaseItems);

                        //Guide message
                        showMessage(R.string.press_to_consume);
                    }
                }
            }
        });
    }

    /**
     * 미처리된 결제건(IAP 서버 검증 실패)들에 대해 일괄로 재처리 작업을 진행합니다.
     * 재처리 결과에 대하여 성공/실패 리스트를 받으며 재처리 대상이 없다면 Array 크기는 0입니다.
     */
    public void clickProcessIncompletePurchases(View view) {
        showLoadingScreen(true);

        InAppPurchases.InAppPurchase.processesIncompletePurchases(this, new InAppPurchase.IncompletePurchasesCallback() {
            @Override
            public void onCallback(JSONObject result, InAppPurchaseException exception) {
                showLoadingScreen(false);

                if (exception != null) {
                    handlePurchaseException(exception);
                } else {
                    JSONArray successArray = result.optJSONArray("successList");
                    JSONArray failArray = result.optJSONArray("failList");

                    //TODO : Enter your code
                    String successResultString = jsonArrayToString(successArray);
                    String failResultString = jsonArrayToString(failArray);

                    if (TextUtils.isEmpty(successResultString) && TextUtils.isEmpty(failResultString)) {
                        showMessage(R.string.no_incomplete_purchases);
                    } else {
                        showMessage(
                                (!TextUtils.isEmpty(successResultString) ? getString(R.string.success) + " : " + successResultString : "") +
                                        (!TextUtils.isEmpty(failResultString) ? "\n" + getString(R.string.fail) + " : " + failResultString : ""));
                    }
                }
            }
        });
    }

    /**
     * Payment Consume API
     * 사용자 애플리케이션 서버는 아이템을 지급하기 전에 IAP 서버에게 결제를 소비 할 것을 알려야 합니다.
     * <p>
     * 서버에서는 결제 구매 토큰(Payment Purchase Token)을 이용하여 사용자 서버와 IAP 서버 간의 결제 유효성에 대한 보안을 체크합니다.
     * 결제 1건당 1번만 결제소비 가능하며, 결제의 상태가 정상이 아니면 소비되지 않습니다.
     * 소비 (Consume) 하지 않은 결제내역은 SDK의 미소비 결제 내역조회 (queryPurchases)를 통해 조회가능 합니다.
     *
     * @param purchaseItem 결제 된 아이템 정보
     */
    private void clickConsume(final PurchaseItem purchaseItem) {
        showLoadingScreen(true);

        //TODO : Consume API 는 서버 to 서버 호출을 권장합니다.
        String consumeApiUrl = "https://api-iap.cloud.toast.com/inapp/v3/consume/" + purchaseItem.getPaymentSeq() + "/items/" + purchaseItem.getItemSeq();
        JSONObject body = new JSONObject();
        try {
            body.put("purchaseToken", purchaseItem.getPurchaseToken());
        } catch (JSONException e) {
            e.printStackTrace();
        }

        new Thread(new HttpPostRunnable(
                consumeApiUrl,
                body,
                new HttpPostRunnable.HttpRequestListener() {
                    @Override
                    public void onSuccess() {
                        showLoadingScreen(false);
                        showMessage(purchaseItem.getItemSeq() + " " + getString(R.string.consume_completed));
                    }

                    @Override
                    public void onError(int code, String message) {
                        showLoadingScreen(false);
                        showMessage(message);
                    }
                })).start();
    }

    /**
     * 조회 된 아이템 리스트에서 선택 된 아이템을 Item ID 필드에 기입합니다.
     *
     * @param queryItem 조회 된 아이템 정보
     */
    private void clickQueryItemSelected(QueryItem queryItem) {
        ((EditText) findViewById(R.id.item_id)).setText(String.valueOf(queryItem.getItemSeq()));
    }

    /**
     * 중복 입력을 방지하기 위한 로딩 화면을 보여줍니다.
     *
     * @param show visible 여부
     */
    private void showLoadingScreen(final boolean show) {
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                findViewById(R.id.wait_screen).setVisibility(show ? View.VISIBLE : View.GONE);
            }
        });
    }


    /**
     * InAppPurchaseException 의 에러코드를 확인합니다.
     *
     * @param exception SDK 에서 발생한 예외 메시지
     */
    private void handlePurchaseException(@NonNull InAppPurchaseException exception) {
        int errorCode = exception.getErrorCode();

        if (errorCode == InAppExceptionType.INAPP_ONESTORE_NEED_UPDATE.getErrorCode()) {
            OneStoreAppInstaller.updateOrInstall(this);
        } else {
            showMessage(exception.getMessage());
        }
    }

    /**
     * String Resource ID를 사용하여 토스트 팝업을 출력합니다.
     *
     * @param resId String Resource ID
     */
    private void showMessage(int resId) {
        showMessage(getString(resId));
    }

    /**
     * Main Looper 에서 토스트 팝업을 출력합니다.
     *
     * @param message 보여줄 메시지
     */
    private void showMessage(final String message) {
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                Toast.makeText(getApplicationContext(), message, Toast.LENGTH_LONG).show();
            }
        });
    }

    /**
     * List Alert 을 출력합니다.
     *
     * @param resId Title Resource ID
     * @param items PurchaseItem 또는 QueryItem ArrayList
     */
    private void showListAlert(int resId, final ArrayList<?> items) {
        showListAlert(getString(resId), items);
    }

    /**
     * Main Looper 에서 List Alert 을 출력합니다.
     *
     * @param title Title Message
     * @param items PurchaseItem 또는 QueryItem 에 대한  ArrayList
     */
    private void showListAlert(final String title, final ArrayList<?> items) {
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                showLoadingScreen(false);

                AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(MainActivity.this);
                dialogBuilder.setTitle(title);
                dialogBuilder.setNegativeButton(R.string.com_toast_iap_btn_cancel, null);

                ArrayAdapter<?> adapter = new ArrayAdapter<>(MainActivity.this, R.layout.alert_list_item, items);
                dialogBuilder.setAdapter(adapter,
                        new DialogInterface.OnClickListener() {
                            public void onClick(DialogInterface dialog, int id) {
                                Object item = items.get(id);

                                if (item instanceof QueryItem) {
                                    //QueryItem 일 경우 아이템 정보 기입
                                    clickQueryItemSelected((QueryItem) item);
                                } else if (item instanceof PurchaseItem) {
                                    //PurchaseItem 일 경우 소비 진행
                                    clickConsume((PurchaseItem) item);
                                }
                            }
                        });

                int padding = (int) getResources().getDimension(R.dimen.dialog_padding);
                AlertDialog alertDialog = dialogBuilder.create();
                ListView listView = alertDialog.getListView();
                //List Divider
                listView.setDivider(new ColorDrawable(Color.BLACK));
                listView.setDividerHeight(2);
                listView.setPadding(padding, padding, padding, padding);

                alertDialog.show();
            }
        });
    }

    /**
     * JSON Array 를 임의로 정의한 string message 로 변환합니다.
     *
     * @param jsonArray json 형태의 array list
     * @return 변환 된 String message, itemID(ItemSeq)..N
     */
    private String jsonArrayToString(JSONArray jsonArray) {
        StringBuilder resultStringBuilder = new StringBuilder();

        if (jsonArray.length() > 0) {
            resultStringBuilder.append(jsonArray.length()).append('\n');

            for (int i = 0; i < jsonArray.length(); i++) {
                JSONObject payment = jsonArray.optJSONObject(i);
                PurchaseItem purchaseItem = new PurchaseItem(payment);
                resultStringBuilder.append(String.format("%s(%s)\n", purchaseItem.getMarketItemId(), purchaseItem.getItemSeq()));
            }
        }

        return resultStringBuilder.toString();
    }
}
